/**
 * Partials for the Chart.js configuration.
 */
window.daextdaucConfigPartials = {

  /**
   * Display an half Pie/Doughnut if enabled with the Half Pie/Doughnut option.
   *
   * @param chart
   * @returns {{}}
   */
  addHalfPieConfiguration: function(currentConfig, chart) {

    'use strict';

    if ((chart.type == 'pie' || chart.type == 'doughnut') &&
        parseInt(chart.half_pie_doughnut, 10) === 1) {

      let newConfig = {};

      newConfig.rotation = 1 * Math.PI;
      newConfig.circumference = 1 * Math.PI;

      return newConfig;

    } else {

      return currentConfig;

    }

  },

  /**
   * Add the legend configuration.
   *
   * @param currentConfig
   * @param chart
   * @returns {any}
   */
  addLegendConfiguration: function(currentConfig, chart){

    'use strict';

    //Make the legend clickable or disabled
    if (parseInt(chart.legend_toggle_dataset, 10) === 0) {

      return Object.assign(currentConfig,
          {
            legend: {
              onClick: function() {
                return;
              },
            },
          },
      );

    }else{

      return currentConfig;

    }

  },

  /**
   * Add the tooltips configuration.
   *
   * @param currentConfig
   * @param chart
   * @returns {any}
   */
  addTooltipsConfiguration: function(currentConfig, chart) {

    'use strict';

    let callbacks = {};

    if (chart.tooltips_beforeTitle.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        beforeTitle: function() {
          return chart.tooltips_beforeTitle;
        },
      });
    }

    if (chart.tooltips_afterTitle.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        afterTitle: function() {
          return chart.tooltips_afterTitle;
        },
      });
    }

    if (chart.tooltips_beforeBody.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        beforeBody: function() {
          return chart.tooltips_beforeBody;
        },
      });
    }

    if (chart.tooltips_afterBody.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        afterBody: function() {
          return chart.tooltips_afterBody;
        },
      });
    }

    if (chart.tooltips_beforeLabel.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        beforeLabel: function() {
          return chart.tooltips_beforeLabel;
        },
      });
    }

    if (chart.tooltips_afterLabel.trim().length > 0) {
      callbacks = Object.assign(callbacks, {
        afterLabel: function() {
          return chart.tooltips_afterLabel;
        },
      });
    }

    callbacks = Object.assign(callbacks, {
      label: function(tooltipItems, data) {

        'use strict';

        let value = null;

        switch (chart.type) {

          case 'horizontalBar':

            value = daextdaucUtility.formatValue(chart,
                data.datasets[tooltipItems.datasetIndex].data[tooltipItems.index]);
            return data.datasets[tooltipItems.datasetIndex].label + ': ' +
                chart.tooltips_beforeValue + value + chart.tooltips_afterValue;

            break;

          case 'polarArea':

            value = daextdaucUtility.formatValue(chart, tooltipItems.yLabel);
            return data.labels[tooltipItems.index] + ': ' +
                chart.tooltips_beforeValue + value + chart.tooltips_afterValue;

            break;

          case 'doughnut':
          case 'pie':

            value = daextdaucUtility.formatValue(chart,
                data.datasets[tooltipItems.datasetIndex].data[tooltipItems.index]);
            return data.labels[tooltipItems.index] + ': ' +
                chart.tooltips_beforeValue + value + chart.tooltips_afterValue;

            break;

          case 'bubble':

            const valueX = daextdaucUtility.formatValue(chart,
                data.datasets[tooltipItems.datasetIndex].data[tooltipItems.index].x);
            const valueY = daextdaucUtility.formatValue(chart,
                data.datasets[tooltipItems.datasetIndex].data[tooltipItems.index].y);
            const valueR = daextdaucUtility.formatValue(chart,
                data.datasets[tooltipItems.datasetIndex].data[tooltipItems.index].r);
            return data.datasets[tooltipItems.datasetIndex].label + ': ' +
                chart.tooltips_beforeValue + '(' + valueX + ', ' + valueY +
                ', ' + valueR + ')' + chart.tooltips_afterValue;

            break;

          default:

            value = daextdaucUtility.formatValue(chart, tooltipItems.yLabel);
            return data.datasets[tooltipItems.datasetIndex].label + ': ' +
                chart.tooltips_beforeValue + value + chart.tooltips_afterValue;

            break;

        }

      },
    });

    if (chart.tooltips_beforeFooter.trim().length > 0) {

      callbacks = Object.assign(callbacks, {
        beforeFooter: function() {

          'use strict';

          return chart.tooltips_beforeFooter;

        },
      });

    }

    if (chart.tooltips_footer.trim().length > 0) {

      callbacks = Object.assign(callbacks, {
        footer: function() {

          'use strict';

          return chart.tooltips_footer;

        },
      });

    }

    if (chart.tooltips_afterFooter.trim().length > 0) {

      callbacks = Object.assign(callbacks, {
        afterFooter: function() {

          'use strict';

          return chart.tooltips_afterFooter;

        },
      });

    }

    return Object.assign(currentConfig,
        {
          tooltips: {
            callbacks: callbacks,
          }
        }
    );

  },

  /**
   * Enable or disable Animation Rotate and Animation Scale for polar area and
   * doughnut.
   *
   * @param currentConfig
   * @param chart
   * @returns {any}
   */
  addAnimationConfiguration: function(currentConfig, chart){

    'use strict';

    if (chart.type == 'polarArea' || chart.type == 'doughnut') {

      return Object.assign(currentConfig,
          {
            animation: {
              animateRotate: daextdaucUtility.booleanString(
                  chart.animation_animateRotate),
              animateScale: daextdaucUtility.booleanString(
                  chart.animation_animateScale),
            },
          },
      );

    }else{

      return currentConfig;

    }

  },

  /**
   * Set the side of the skipped border based on the dedicated option. Note that
   * this settings affects the bar and horizontalBar charts.
   *
   * @param currentConfig
   * @param chart
   */
  addElementsConfiguration: function (currentConfig, chart){

    'use strict';

    if (chart.type == 'bar' || chart.type == 'horizontalBar') {

      return Object.assign(currentConfig, {
        elements: {
          rectangle: {
            borderSkipped: chart.elements_rectangle_borderSkipped,
          },
        },
      });

    }else{

      return currentConfig;

    }

  },

  /**
   * Add the settings for the radial scale.
   *
   * @param currentConfig
   * @param chart
   * @returns {any}
   */
  addScaleRlConfiguration: function(currentConfig, chart){

    'use strict';

    const scale = {};
    scale.display = false;

    if ((chart.type == 'radar' || chart.type == 'polarArea') &&
    parseInt(chart.scales_rl_display, 10) === 1) {

      //RL Scale Common
      scale.display = true;

      //R Scale Grid Line
      scale.gridLines = {
        display: daextdaucUtility.booleanString(
            chart.scales_rl_gridLines_display),
        color: daextdaucUtility.outputColor(
            chart.scales_rl_gridLines_color, chart.type,
            'scales_rl_gridLines_color'),
        lineWidth: daextdaucUtility.outputNumber(
            chart.scales_rl_gridLines_lineWidth, chart.type,
            'scales_rl_gridLines_lineWidth'),
      };

      //RL Scale Angle Line
      scale.angleLines = {
        display: daextdaucUtility.booleanString(
            chart.scales_rl_angleLines_display),
        color: daextdaucUtility.outputColor(
            chart.scales_rl_angleLines_color, chart.type,
            'scales_rl_angleLines_color'),
        lineWidth: daextdaucUtility.outputNumber(
            chart.scales_rl_angleLines_lineWidth, chart.type,
            'scales_rl_angleLines_lineWidth'),
      };

      //RL Scale Point Label
      scale.pointLabels = {
        fontSize: parseInt(chart.scales_rl_pointLabels_fontSize, 10),
        fontColor: daextdaucUtility.outputColor(
            chart.scales_rl_pointLabels_fontColor, chart.type,
            'scales_rl_pointLabels_fontColor'),
        fontFamily: chart.scales_rl_pointLabels_fontFamily,
        fontStyle: chart.scales_rl_pointLabels_fontStyle,
      };

      //RL Scale Tick
      scale.reverse = daextdaucUtility.booleanString(
          chart.scales_rl_ticks_reverse),
          scale.ticks = {

            display: daextdaucUtility.booleanString(
                chart.scales_rl_ticks_display),
            autoskip: daextdaucUtility.booleanString(
                chart.scales_rl_ticks_autoskip),
            fontSize: parseInt(chart.scales_rl_ticks_fontSize, 10),
            fontColor: daextdaucUtility.outputColor(
                chart.scales_rl_ticks_fontColor, chart.type,
                'scales_rl_ticks_fontColor'),
            fontFamily: chart.scales_rl_ticks_fontFamily,
            fontStyle: chart.scales_rl_ticks_fontStyle,

          };

      //RL Scale Options (specific for the radial linear scale)
      if (chart.scales_rl_ticks_min.trim().length > 0) {
        scale.ticks.min = chart.scales_rl_ticks_min;
      }
      if (chart.scales_rl_ticks_max.trim().length > 0) {
        scale.ticks.max = parseFloat(chart.scales_rl_ticks_max);
      }
      if (chart.scales_rl_ticks_suggestedMin.trim().length) {
        scale.ticks.suggestedMin = parseFloat(
            chart.scales_rl_ticks_suggestedMin);
      }
      if (chart.scales_rl_ticks_suggestedMax.trim().length > 0) {
        scale.ticks.suggestedMax = parseFloat(
            chart.scales_rl_ticks_suggestedMax);
      }
      if (chart.scales_rl_ticks_stepSize.trim().length > 0) {
        scale.ticks.stepSize = parseFloat(chart.scales_rl_ticks_stepSize);
      }
      if (chart.scales_rl_ticks_fixedStepSize > 0) {
        scale.ticks.fixedStepSize = parseFloat(
            chart.scales_rl_ticks_fixedStepSize);
      }
      if (chart.scales_rl_ticks_maxTicksLimit.trim().length > 0) {
        scale.ticks.maxTicksLimit = parseInt(
            chart.scales_rl_ticks_maxTicksLimit, 10);
      }
      scale.ticks.beginAtZero = daextdaucUtility.booleanString(
          chart.scales_rl_ticks_beginAtZero);
      scale.ticks.showLabelBackdrop = daextdaucUtility.booleanString(
          chart.scales_rl_ticks_showLabelBackdrop);
      scale.ticks.backdropColor = daextdaucUtility.outputColor(
          chart.scales_rl_ticks_backdropColor, chart.type,
          'scales_rl_ticks_backdropColor');
      scale.ticks.backdropPaddingX = parseInt(
          chart.scales_rl_ticks_backdropPaddingX, 10);
      scale.ticks.backdropPaddingY = parseInt(
          chart.scales_rl_ticks_backdropPaddingY, 10);

      scale.ticks.userCallback = function(tick) {

        'use strict';

        let tick_output = null;

        if (chart.scales_rl_ticks_round.trim().length > 0) {
          tick_output = tick.toFixed(
              parseInt(chart.scales_rl_ticks_round, 10)).toString();
        } else {
          tick_output = tick.toString();
        }

        tick_output = daextdaucUtility.formatValue(chart, tick_output);

        return chart.scales_rl_ticks_prefix + tick_output +
            chart.scales_rl_ticks_suffix;

      };

    }

    return Object.assign(currentConfig, {
      scale: scale
    });

  },

  /**
   * Add the settings for the scale of the X axis.
   *
   * @param chart
   * @returns {{}}
   */
  generateScaleXAxis: function(chart){

    'use strict';

    let config = {};

    //scales_xAxes_display
    if (parseInt(chart.scales_xAxes_display, 10)) {

      config.display = true;

      //scales_xAxes_type
      config.type = chart.scales_xAxes_type;

      //scales_xAxes_position
      config.position = chart.scales_xAxes_position;

      //scales_xAxes_stacked
      config.stacked = daextdaucUtility.booleanString(
          chart.scales_xAxes_stacked);

      if (chart.type == 'bar') {
        config.categoryPercentage = parseFloat(
            chart.scales_xAxes_categoryPercentage);
        config.barPercentage = parseFloat(
            chart.scales_xAxes_barPercentage);
      }
      ;

      //Scale Grid Line config scales_xAxes_gridLines -----------------------------
      config.gridLines = {
        display: daextdaucUtility.booleanString(
            chart.scales_xAxes_gridLines_display),
        color: daextdaucUtility.outputColor(
            chart.scales_xAxes_gridLines_color, chart.type,
            'scales_xAxes_gridLines_color'),
        lineWidth: daextdaucUtility.outputNumber(
            chart.scales_xAxes_gridLines_lineWidth, chart.type,
            'scales_xAxes_gridLines_lineWidth'),
        drawBorder: daextdaucUtility.booleanString(
            chart.scales_xAxes_gridLines_drawBorder),
        drawOnChartArea: daextdaucUtility.booleanString(
            chart.scales_xAxes_gridLines_drawOnChartArea),
        drawTicks: daextdaucUtility.booleanString(
            chart.scales_xAxes_gridLines_drawTicks),
        tickMarkLength: parseInt(
            chart.scales_xAxes_gridLines_tickMarkLength, 10),
        zeroLineWidth: parseInt(
            chart.scales_xAxes_gridLines_zeroLineWidth, 10),
        zeroLineColor: daextdaucUtility.outputColor(
            chart.scales_xAxes_gridLines_zeroLineColor, chart.type,
            'scales_xAxes_gridLines_zeroLineColor'),
        offsetGridLines: daextdaucUtility.booleanString(
            chart.scales_xAxes_gridLines_offsetGridLines),
      };

      //Scale Title config scales_xAxes_scaleLabel --------------------------------
      config.scaleLabel = {
        display: daextdaucUtility.booleanString(
            chart.scales_xAxes_scaleLabel_display),
        labelString: chart.scales_xAxes_scaleLabel_labelString,
        fontColor: daextdaucUtility.outputColor(
            chart.scales_xAxes_scaleLabel_fontColor, chart.type,
            'scales_xAxes_scaleLabel_fontColor'),
        fontFamily: chart.scales_xAxes_scaleLabel_fontFamily,
        fontSize: parseInt(chart.scales_xAxes_scaleLabel_fontSize, 10),
        fontStyle: chart.scales_xAxes_scaleLabel_fontStyle,
      };

      //Scale Tick config scales_xAxes_ticks --------------------------------------
      config.ticks = {};

      /**
       * If there is a logarithmic scale active this function is used to display the real
       * value on the xAxis.
       * Note that this function with this specific content should be used only with the
       * logarithmic scale. Other versions of this function might be used to customize the
       * string representation of the tick value
       */
      if (chart.scales_xAxes_type == 'logarithmic') {

        config.ticks.userCallback = function(tick) {

          'use strict';

          let tick_output = null;
          const remain = tick /
              (Math.pow(10, Math.floor(Chart.helpers.log10(tick))));

          //this determines how many ticks are shown
          if (remain === 1 || remain === 2 || remain === 5) {

            if (chart.scales_xAxes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chart.scales_xAxes_ticks_round, 10)).
                  toString();

            } else {
              tick_output = tick.toString();

            }

            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);

            return chart.scales_xAxes_ticks_prefix + tick_output +
                chart.scales_xAxes_ticks_suffix;

          }

          return '';

        };

      } else {

        config.ticks.userCallback = function(tick) {

          let tick_output = null;

          if (chart.scales_xAxes_type == 'linear') {

            if (chart.scales_xAxes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chart.scales_xAxes_ticks_round, 10)).
                  toString();

            } else {
              tick_output = tick.toString();

            }

            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);

          } else {
            tick_output = tick.toString();
          }

          return chart.scales_xAxes_ticks_prefix + tick_output +
              chart.scales_xAxes_ticks_suffix;

        };

      }

      //Scale Tick config -----------------------------------------------------
      config.ticks.autoSkip = daextdaucUtility.booleanString(
          chart.scales_xAxes_ticks_autoskip);
      config.ticks.display = daextdaucUtility.booleanString(
          chart.scales_xAxes_ticks_display);
      config.ticks.fontColor = daextdaucUtility.outputColor(
          chart.scales_xAxes_ticks_fontColor, chart.type,
          'scales_xAxes_ticks_fontColor');
      config.ticks.fontFamily = chart.scales_xAxes_ticks_fontFamily;
      config.ticks.fontSize = parseInt(
          chart.scales_xAxes_ticks_fontSize, 10);
      config.ticks.fontStyle = chart.scales_xAxes_ticks_fontStyle;
      config.ticks.labelOffset = parseInt(
          chart.scales_xAxes_ticks_labelOffset, 10);
      config.ticks.maxRotation = parseInt(
          chart.scales_xAxes_ticks_maxRotation, 10);
      config.ticks.minRotation = parseInt(
          chart.scales_xAxes_ticks_minRotation, 10);
      config.ticks.reverse = daextdaucUtility.booleanString(
          chart.scales_xAxes_ticks_reverse);

      //Scale config Options --------------------------------------------------
      if (chart.scales_xAxes_ticks_min.trim().length > 0) {
        config.ticks.min = parseFloat(chart.scales_xAxes_ticks_min);
      }
      if (chart.scales_xAxes_ticks_max.trim().length > 0) {
        config.ticks.max = parseFloat(chart.scales_xAxes_ticks_max);
      }

      //Scale Linear config Options -------------------------------------------
      if (chart.scales_xAxes_type == 'linear') {

        if (chart.scales_xAxes_ticks_fixedStepSize.trim().length > 0) {
          config.ticks.fixedStepSize = parseFloat(
              chart.scales_xAxes_ticks_fixedStepSize);
        }
        config.ticks.beginAtZero = daextdaucUtility.booleanString(
            chart.scales_xAxes_ticks_beginAtZero);
        if (chart.scales_xAxes_ticks_maxTicksLimit.trim().length > 0) {
          config.ticks.maxTicksLimit = parseInt(
              chart.scales_xAxes_ticks_maxTicksLimit, 10);
        }
        if (chart.scales_xAxes_ticks_stepSize.trim().length > 0) {
          config.ticks.stepSize = parseFloat(
              chart.scales_xAxes_ticks_stepSize);
        }
        if (chart.scales_xAxes_ticks_suggestedMax.trim().length > 0) {
          config.ticks.suggestedMax = parseFloat(
              chart.scales_xAxes_ticks_suggestedMax);
        }
        if (chart.scales_xAxes_ticks_suggestedMin.trim().length > 0) {
          config.ticks.suggestedMin = parseFloat(
              chart.scales_xAxes_ticks_suggestedMin);
        }

      }

      //Scale Time config Options ------------------------------------------------------
      if (chart.scales_xAxes_type == 'time') {

        config.time = {};

        config.time.format = chart.scales_xAxes_time_format;
        config.time.tooltipFormat = chart.scales_xAxes_time_tooltipFormat;
        config.time.unitStepSize = parseInt(
            chart.scales_xAxes_time_unitStepSize, 10);

        if (chart.scales_xAxes_time_max.trim().length > 0) {
          config.time.max = chart.scales_xAxes_time_max;
        }

        if (chart.scales_xAxes_time_min.trim().length > 0) {
          config.time.min = chart.scales_xAxes_time_min;
        }

        config.time.displayFormats = {
          [chart.scales_xAxes_time_unit]: chart.scales_xAxes_time_unit_format
        };

        config.time.unit = chart.scales_xAxes_time_unit;

      }

    } else {

      config.display = false;

    }

    return config;

  },

  /**
   * Add the settings for the scale of the Y Axis.
   *
   * @param chart
   * @returns {{}}
   */
  generateScaleYAxis: function(chart){

    'use strict';

    const config = {};

    if (parseInt(chart.scales_yAxes_display, 10) === 1) {

      config.display = true;

      //scales_yAxes_type
      config.type = chart.scales_yAxes_type;

      //scales_yAxes_position
      config.position = chart.scales_yAxes_position;

      //scales_yAxes_stacked
      config.stacked = daextdaucUtility.booleanString(
          chart.scales_yAxes_stacked);//default: false

      if (chart.type == 'horizontalBar') {

        config.categoryPercentage = parseFloat(
            chart.scales_yAxes_categoryPercentage);
        config.barPercentage = parseFloat(
            chart.scales_yAxes_barPercentage);

      }

      //Scale Grid Line Configuration scales_yAxes_gridLines -----------------------------
      config.gridLines = {
        display: daextdaucUtility.booleanString(
            chart.scales_yAxes_gridLines_display),
        color: daextdaucUtility.outputColor(
            chart.scales_yAxes_gridLines_color, chart.type,
            'scales_yAxes_gridLines_color'),
        lineWidth: daextdaucUtility.outputNumber(
            chart.scales_yAxes_gridLines_lineWidth, chart.type,
            'scales_yAxes_gridLines_lineWidth'),
        drawBorder: daextdaucUtility.booleanString(
            chart.scales_yAxes_gridLines_drawBorder),
        drawOnChartArea: daextdaucUtility.booleanString(
            chart.scales_yAxes_gridLines_drawOnChartArea),
        drawTicks: daextdaucUtility.booleanString(
            chart.scales_yAxes_gridLines_drawTicks),
        tickMarkLength: parseInt(
            chart.scales_yAxes_gridLines_tickMarkLength, 10),
        zeroLineWidth: parseInt(
            chart.scales_yAxes_gridLines_zeroLineWidth, 10),
        zeroLineColor: daextdaucUtility.outputColor(
            chart.scales_yAxes_gridLines_zeroLineColor, chart.type,
            'scales_yAxes_gridLines_zeroLineColor'),
        offsetGridLines: daextdaucUtility.booleanString(
            chart.scales_yAxes_gridLines_offsetGridLines),
      };

      //Scale Title Configuration scales_yAxes_scaleLabel --------------------------------
      config.scaleLabel = {
        display: daextdaucUtility.booleanString(
            chart.scales_yAxes_scaleLabel_display),
        labelString: chart.scales_yAxes_scaleLabel_labelString,
        fontColor: daextdaucUtility.outputColor(
            chart.scales_yAxes_scaleLabel_fontColor, chart.type,
            'scales_yAxes_scaleLabel_fontColor'),
        fontFamily: chart.scales_yAxes_scaleLabel_fontFamily,
        fontSize: parseInt(chart.scales_yAxes_scaleLabel_fontSize, 10),
        fontStyle: chart.scales_yAxes_scaleLabel_fontStyle,
      };

      //Scale Tick Configuration scales_yAxes_ticks --------------------------------------
      config.ticks = {};

      /**
       * If there is a logarithmic scale active this function is used to display the real
       * value on the yAxis.
       * Note that this function with this specific content should be used only with the
       * logarithmic scales. Other versions of this function might be used to customize the
       * string representation of the tick value.
       */

      if (chart.scales_yAxes_type == 'logarithmic') {

        config.ticks.userCallback = function(tick) {

          let tick_output = null;

          const remain = tick /
              (Math.pow(10, Math.floor(Chart.helpers.log10(tick))));

          //this determines how many ticks are shown
          if (remain === 1 || remain === 2 || remain === 5) {

            if (chart.scales_yAxes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chart.scales_yAxes_ticks_round, 10)).
                  toString();

            } else {
              tick_output = tick.toString();

            }

            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);

            return chart.scales_yAxes_ticks_prefix + tick_output +
                chart.scales_yAxes_ticks_suffix;

          }

          return '';

        };

      } else {

        config.ticks.userCallback = function(tick) {

          'use strict';

          let tick_output;

          if (chart.scales_yAxes_type == 'linear') {
            if (chart.scales_yAxes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chart.scales_yAxes_ticks_round, 10)).
                  toString();

            } else {
              tick_output = tick.toString();
            }
            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);
          } else {
            tick_output = tick.toString();
          }

          return chart.scales_yAxes_ticks_prefix + tick_output +
              chart.scales_yAxes_ticks_suffix;

        };

      }

      //Scale Tick Configuration -----------------------------------------------------
      config.ticks.autoSkip = daextdaucUtility.booleanString(
          chart.scales_yAxes_ticks_autoskip);
      config.ticks.display = daextdaucUtility.booleanString(
          chart.scales_yAxes_ticks_display);
      config.ticks.fontColor = daextdaucUtility.outputColor(
          chart.scales_yAxes_ticks_fontColor, chart.type,
          'scales_yAxes_ticks_fontColor'),
          config.ticks.fontFamily = chart.scales_yAxes_ticks_fontFamily;
      config.ticks.fontSize = parseInt(
          chart.scales_yAxes_ticks_fontSize, 10);
      config.ticks.fontStyle = chart.scales_yAxes_ticks_fontStyle;
      config.ticks.maxRotation = parseInt(
          chart.scales_yAxes_ticks_maxRotation, 10);
      config.ticks.minRotation = parseInt(
          chart.scales_yAxes_ticks_minRotation, 10);
      config.ticks.mirror = daextdaucUtility.booleanString(
          chart.scales_yAxes_ticks_mirror);
      config.ticks.padding = parseInt(
          chart.scales_yAxes_ticks_padding, 10);
      config.ticks.reverse = daextdaucUtility.booleanString(
          chart.scales_yAxes_ticks_reverse);

      //Scale Configuration Options --------------------------------------------------
      if (chart.scales_yAxes_ticks_min.trim().length > 0) {
        config.ticks.min = parseInt(
            chart.scales_yAxes_ticks_min);
      }
      if (chart.scales_yAxes_ticks_max.trim().length > 0) {
        config.ticks.max = parseInt(
            chart.scales_yAxes_ticks_max);
      }

      //Scale Linear Configuration Options -------------------------------------------
      if (chart.scales_yAxes_type == 'linear') {

        if (chart.scales_yAxes_ticks_fixedStepSize.trim().length > 0) {
          config.ticks.fixedStepSize = parseFloat(
              chart.scales_yAxes_ticks_fixedStepSize);
        }

        config.ticks.beginAtZero = daextdaucUtility.booleanString(
            chart.scales_yAxes_ticks_beginAtZero);

        if (chart.scales_yAxes_ticks_maxTicksLimit.trim().length > 0) {
          config.ticks.maxTicksLimit = parseInt(
              chart.scales_yAxes_ticks_maxTicksLimit, 10);
        }

        if (chart.scales_yAxes_ticks_stepSize.trim().length > 0) {
          config.ticks.stepSize = parseFloat(
              chart.scales_yAxes_ticks_stepSize);
        }

        if (chart.scales_yAxes_ticks_suggestedMax.trim().length > 0) {
          config.ticks.suggestedMax = parseFloat(
              chart.scales_yAxes_ticks_suggestedMax);
        }

        if (chart.scales_yAxes_ticks_suggestedMin.trim().length > 0) {
          config.ticks.suggestedMin = parseFloat(
              chart.scales_yAxes_ticks_suggestedMin);
        }

      }

    } else {

      config.display = false;

    }

    return config;

  },

  /**
   * Add the settings for the scale of the Y2 axis.
   *
   * @param chart
   * @returns {{}}
   */
  generateScaleY2Axis: function(chart){

    'use strict';

    const config = {};

    if (parseInt(chart.scales_y2Axes_display, 10) === 1) {

      config.display = true;

      //scales_y2Axes_type
      config.type = chart.scales_y2Axes_type;

      //scales_y2Axes_position
      config.position = chart.scales_y2Axes_position;

      //identifier of the Y2 axis
      config.id = 'y2-identifier';

      //Scale Grid Line Configuration scales_y2Axes_gridLines ----------------------------
      config.gridLines = {
        display: daextdaucUtility.booleanString(
            chart.scales_y2Axes_gridLines_display),
        color: daextdaucUtility.outputColor(
            chart.scales_y2Axes_gridLines_color, chart.type,
            'scales_y2Axes_gridLines_color'),
        lineWidth: daextdaucUtility.outputNumber(
            chart.scales_y2Axes_gridLines_lineWidth, chart.type,
            'scales_y2Axes_gridLines_lineWidth'),
        drawBorder: daextdaucUtility.booleanString(
            chart.scales_y2Axes_gridLines_drawBorder),
        drawOnChartArea: daextdaucUtility.booleanString(
            chart.scales_y2Axes_gridLines_drawOnChartArea),
        drawTicks: daextdaucUtility.booleanString(
            chart.scales_y2Axes_gridLines_drawTicks),
        tickMarkLength: parseInt(
            chart.scales_y2Axes_gridLines_tickMarkLength, 10),
        zeroLineWidth: parseInt(
            chart.scales_y2Axes_gridLines_zeroLineWidth, 10),
        zeroLineColor: daextdaucUtility.outputColor(
            chart.scales_y2Axes_gridLines_zeroLineColor, chart.type,
            'scales_y2Axes_gridLines_zeroLineColor'),
        offsetGridLines: daextdaucUtility.booleanString(
            chart.scales_y2Axes_gridLines_offsetGridLines),
      };

      //Scale Title Configuration scales_y2Axes_scaleLabel -------------------------------
      config.scaleLabel = {
        display: daextdaucUtility.booleanString(
            chart.scales_y2Axes_scaleLabel_display),
        labelString: chart.scales_y2Axes_scaleLabel_labelString,
        fontColor: daextdaucUtility.outputColor(
            chart.scales_y2Axes_scaleLabel_fontColor, chart.type,
            'scales_y2Axes_scaleLabel_fontColor'),
        fontFamily: chart.scales_y2Axes_scaleLabel_fontFamily,
        fontSize: parseInt(chart.scales_y2Axes_scaleLabel_fontSize, 10),
        fontStyle: chart.scales_y2Axes_scaleLabel_fontStyle,
      };

      //Scale Tick Configuration scales_y2Axes_ticks -------------------------------------
      config.ticks = {};

      /**
       * If there is a logarithmic scale active this function is used to display the real
       * value on the y2Axis.
       * Note that this function with this specific content should be used only with the
       * logarithmic scales. Other versions of this function might be used to customize the
       * string representation of the tick value.
       */
      if (chart.scales_y2Axes_type == 'logarithmic') {

        config.ticks.userCallback = function(tick) {

          let tick_output = null;
          const remain = tick /
              (Math.pow(10, Math.floor(Chart.helpers.log10(tick))));

          //this determines how many ticks are shown
          if (remain === 1 || remain === 2 || remain === 5) {

            if (chart.scales_y2Axes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chart.scales_y2Axes_ticks_round, 10)).
                  toString();
            } else {
              tick_output = tick.toString();
            }

            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);

            return chart.scales_y2Axes_ticks_prefix + tick_output +
                chart.scales_y2Axes_ticks_suffix;

          }
          return '';
        };

      } else {

        config.ticks.userCallback = function(tick) {

          let tick_output = null;

          if (chart.scales_y2Axes_type == 'linear') {

            if (chart.scales_y2Axes_ticks_round.trim().length > 0) {
              tick_output = tick.toFixed(
                  parseInt(chartscales_y2Axes_ticks_round, 10)).
                  toString();
            } else {
              tick_output = tick.toString();
            }

            tick_output = daextdaucUtility.formatValue(chart,
                tick_output);

          } else {

            tick_output = tick.toString();

          }

          return chart.scales_y2Axes_ticks_prefix + tick_output +
              chart.scales_y2Axes_ticks_suffix;

        };

      }

      //Scale Tick Configuration -----------------------------------------------------
      config.ticks.autoSkip = daextdaucUtility.booleanString(
          chart.scales_y2Axes_ticks_autoskip);
      config.ticks.display = daextdaucUtility.booleanString(
          chart.scales_y2Axes_ticks_display);
      config.ticks.fontColor = daextdaucUtility.outputColor(
          chart.scales_y2Axes_ticks_fontColor, chart.type,
          'scales_y2Axes_ticks_fontColor');
      config.ticks.fontFamily = chart.scales_y2Axes_ticks_fontFamily;
      config.ticks.fontSize = parseInt(
          chart.scales_y2Axes_ticks_fontSize, 10);
      config.ticks.fontStyle = chart.scales_y2Axes_ticks_fontStyle;
      config.ticks.maxRotation = parseInt(
          chart.scales_y2Axes_ticks_maxRotation, 10);
      config.ticks.minRotation = parseInt(
          chart.scales_y2Axes_ticks_minRotation, 10);
      config.ticks.mirror = daextdaucUtility.booleanString(
          chart.scales_y2Axes_ticks_mirror);
      config.ticks.padding = parseInt(
          chart.scales_y2Axes_ticks_padding, 10);
      config.ticks.reverse = daextdaucUtility.booleanString(
          chart.scales_y2Axes_ticks_reverse);

      //Scale Configuration Options --------------------------------------------------
      if (chart.scales_y2Axes_ticks_min.trim().length > 0) {
        config.ticks.min = parseFloat(
            chart.scales_y2Axes_ticks_min);
      }

      if (chart.scales_y2Axes_ticks_max.trim().length > 0) {
        config.ticks.max = parseFloat(
            chart.scales_y2Axes_ticks_max);
      }

      //Scale Linear Configuration Options -------------------------------------------
      if (chart.scales_y2Axes_type == 'linear') {

        if (chart.scales_y2Axes_ticks_fixedStepSize.trim().length > 0) {
          config.ticks.fixedStepSize = parseFloat(
              chart.scales_y2Axes_ticks_fixedStepSize);
        }

        config.ticks.beginAtZero = daextdaucUtility.booleanString(
            chart.scales_y2Axes_ticks_beginAtZero);

        if (chart.scales_y2Axes_ticks_maxTicksLimit.trim().length > 0) {
          config.ticks.maxTicksLimit = parseInt(
              chart.scales_y2Axes_ticks_maxTicksLimit, 10);
        }

        if (chart.scales_y2Axes_ticks_stepSize.trim().length > 0) {
          config.ticks.stepSize = parseFloat(
              chart.scales_y2Axes_ticks_stepSize);
        }

        if (chart.scales_y2Axes_ticks_suggestedMax.trim().length > 0) {
          config.ticks.suggestedMax = parseFloat(
              chart.scales_y2Axes_ticks_suggestedMax);
        }

        if (chart.scales_y2Axes_ticks_suggestedMin.trim().length > 0) {
          config.ticks.suggestedMin = parseFloat(
              chart.scales_y2Axes_ticks_suggestedMin);
        }

      }

    } else {

      config.display = false;

    }

    return config;

  }

};